@@ -2,12 +2,10 @@ package Brackup::Target::Amazon;
 use strict;
 use warnings;
 use base 'Brackup::Target';
-use Net::Amazon::S3 0.31;
-use DBD::SQLite;
+use Net::Amazon::S3 0.37;
 
 # fields in object:
 #   s3  -- Net::Amazon::S3
-#   dbh -- sqlite dbi exist cache
 #   access_key_id
 #   sec_access_key_id
 #   chunk_bucket : $self->{access_key_id} . "-chunks";
@@ -16,24 +14,14 @@ use DBD::SQLite;
 
 sub new {
     my ($class, $confsec) = @_;
-    my $self = bless {}, $class;
-
-    if (my $cache_file = $confsec->value("exist_cache")) {
-        $self->{dbh} = DBI->connect("dbi:SQLite:dbname=$cache_file","","", { RaiseError => 1, PrintError => 0 }) or
-            die "Failed to connect to SQLite filesystem digest cache database at $cache_file: " . DBI->errstr;
-
-        eval {
-            $self->{dbh}->do("CREATE TABLE amazon_key_exists (key TEXT PRIMARY KEY, value TEXT)");
-        };
-        die "Error: $@" if $@ && $@ !~ /table amazon_key_exists already exists/;
-    }
+    my $self = $class->SUPER::new($confsec);
 
     $self->{access_key_id}     = $confsec->value("aws_access_key_id")
         or die "No 'aws_access_key_id'";
     $self->{sec_access_key_id} = $confsec->value("aws_secret_access_key")
         or die "No 'aws_secret_access_key'";
 
-    $self->_init_common;
+    $self->_common_s3_init;
 
     my $s3      = $self->{s3};
     my $buckets = $s3->buckets or die "Failed to get bucket list";
@@ -51,7 +39,7 @@ sub new {
     return $self;
 }
 
-sub _init_common {
+sub _common_s3_init {
     my $self = shift;
     $self->{chunk_bucket}  = $self->{access_key_id} . "-chunks";
     $self->{backup_bucket} = $self->{access_key_id} . "-backups";
@@ -82,7 +70,7 @@ sub new_from_backup_header {
     my $self = bless {}, $class;
     $self->{access_key_id}     = $accesskey;
     $self->{sec_access_key_id} = $sec_accesskey;
-    $self->_init_common;
+    $self->_common_s3_init;
     return $self;
 }
 
@@ -90,16 +78,10 @@ sub has_chunk {
     my ($self, $chunk) = @_;
     my $dig = $chunk->backup_digest;   # "sha1:sdfsdf" format scalar
 
-    if (my $dbh = $self->{dbh}) {
-        my $ans = $dbh->selectrow_array("SELECT COUNT(*) FROM amazon_key_exists WHERE key=?", undef, $dig);
-        return 1 if $ans;
-    }
-
     my $res = eval { $self->{s3}->head_key({ bucket => $self->{chunk_bucket}, key => $dig }); };
     return 0 unless $res;
     return 0 if $@ && $@ =~ /key not found/;
     return 0 unless $res->{content_type} eq "x-danga/brackup-chunk";
-    $self->_cache_existence_of($dig);
     return 1;
 }
 
@@ -117,23 +99,35 @@ sub store_chunk {
     my $dig = $chunk->backup_digest;
     my $blen = $chunk->backup_length;
     my $len = $chunk->length;
+    my $chunkref = $chunk->chunkref;
 
-    my $rv = eval { $self->{s3}->add_key({
-        bucket        => $self->{chunk_bucket},
-        key           => $dig,
-        value         => ${ $chunk->chunkref },
-        content_type  => 'x-danga/brackup-chunk',
-    }) };
-    return 0 unless $rv;
-    $self->_cache_existence_of($dig);
-    return 1;
-}
+    my $try = sub {
+        eval {
+            $self->{s3}->add_key({
+                bucket        => $self->{chunk_bucket},
+                key           => $dig,
+                value         => $$chunkref,
+                content_type  => 'x-danga/brackup-chunk',
+            });
+        };
+    };
 
-sub _cache_existence_of {
-    my ($self, $dig) = @_;
-    if (my $dbh = $self->{dbh}) {
-        $dbh->do("INSERT INTO amazon_key_exists VALUES (?,1)", undef, $dig);
+    my $rv;
+    my $n_fails = 0;
+    while (!$rv && $n_fails < 5) {
+        $rv = $try->();
+        last if $rv;
+
+        # transient failure?
+        $n_fails++;
+        warn "Error uploading chunk $chunk [$@]... will do retry \#$n_fails in 5 seconds ...\n";
+        sleep 5;
     }
+    unless ($rv) {
+        warn "Error uploading chunk again: " . $self->{s3}->errstr . "\n";
+        return 0;
+    }
+    return 1;
 }
 
 sub store_backup_meta {
