@@ -6,8 +6,10 @@ use File::Path;
 
 sub new {
     my ($class, $confsec) = @_;
-    my $self = bless {}, $class;
+    my $self = $class->SUPER::new($confsec);
     $self->{path} = $confsec->path_value("path");
+    $self->{nocolons} = $confsec->value("no_filename_colons");
+    $self->{nocolons} = ($^O eq 'MSWin32') unless defined $self->{nocolons}; # LAME: Make it work on Windows
     return $self;
 }
 
@@ -29,21 +31,32 @@ sub backup_header {
     };
 }
 
-sub chunkpath {
-    my ($self, $dig) = @_;
+sub _diskpath {
+    my ($self, $dig, $ext) = @_;
     my @parts;
     my $fulldig = $dig;
     $dig =~ s/^\w+://; # remove the "hashtype:" from beginning
+    $fulldig =~ s/:/./g if $self->{nocolons}; # Convert colons to dots if we've been asked to
     while (length $dig && @parts < 4) {
         $dig =~ s/^(.{1,4})//;
         push @parts, $1;
     }
-    return $self->{path} . "/" . join("/", @parts) . "/$fulldig.chunk";
+    return $self->{path} . "/" . join("/", @parts) . "/$fulldig.$ext";
 }
 
-sub has_chunk {
-    my ($self, $chunk) = @_;
-    my $dig = $chunk->backup_digest;   # "sha1:sdfsdf" format scalar
+sub chunkpath {
+    my ($self, $dig) = @_;
+    return $self->_diskpath($dig, "chunk");
+}
+
+sub metapath {
+    my ($self, $dig) = @_;
+    return $self->_diskpath($dig, "meta");
+}
+
+sub has_chunk_of_handle {
+    my ($self, $handle) = @_;
+    my $dig = $handle->digest;  # "sha1:sdfsdf" format scalar
     my $path = $self->chunkpath($dig);
     return -e $path;
 }
@@ -69,14 +82,18 @@ sub store_chunk {
         File::Path::mkpath($dir) or die "Failed to mkdir: $dir: $!\n";
     }
     open (my $fh, ">$path") or die "Failed to open $path for writing: $!\n";
+    binmode($fh);
     my $chunkref = $chunk->chunkref;
     print $fh $$chunkref;
     close($fh) or die "Failed to close $path\n";
 
     my $actual_size   = -s $path;
     my $expected_size = length $$chunkref;
+    unless (defined($actual_size)) {
+        die "Chunk output file $path does not exist. Do you need to set no_filename_colons=1?";
+    }
     unless ($actual_size == $expected_size) {
-        die "Chunk was written to disk wrong:  size is $actual_size, expecting $expected_size\n";
+        die "Chunk $path was written to disk wrong:  size is $actual_size, expecting $expected_size\n";
     }
 
     return 1;
